require 'spec_helper'

module Blamescope
  describe Server do
    describe "::start" do
      it "should initialize server if options provided" do
        options = {database: 'db_url', rabbit: 'rabbit_url'}
        expect(Server).to receive_message_chain(:new, :listen).with(options)
        Server.start(options)
      end
      it "shouldn't initialize server if any option missing" do
        expect(Server).to receive(:puts).twice
        expect(Server).not_to receive(:new)
        Server.start(database: 'db_url')
        Server.start(rabbit: 'rabbit_url')
      end
    end

    describe "#listen" do
      it "should listen for events" do
        subject = Server.new
        queue = double('rabbit queue')
        expect(queue).to receive(:subscribe).and_yield(:info, :properties,:body)
        expect(subject).to receive(:rabbit_connection).with(:rabbit).and_return(queue: queue)
        expect(subject).to receive(:postgres_connection).with(:database).and_return(:db)
        expect(subject).to receive(:store_event).with(:db, :body)
        subject.listen(rabbit: :rabbit, database: :database)
      end

      it "should close channel and connection on interrupt" do
        subject = Server.new
        queue = double('rabbit queue')
        expect(queue).to receive(:subscribe).and_raise(Interrupt)
        channel = double('rabbit channel')
        expect(channel).to receive(:close)
        conn = double('rabbit connection')
        expect(conn).to receive(:close)
        expect(subject).to receive(:rabbit_connection).with(:rabbit).and_return(queue: queue, channel: channel, conn: conn)
        expect(subject).to receive(:postgres_connection).with(:database).and_return(:db)
        subject.listen(rabbit: :rabbit, database: :database)
      end
    end
    describe "#store_event" do
      it "should store event" do
        time = Time.now.utc
        db = double('database connection')
        expect(db).to receive(:exec_prepared).with('insert_event',
                             ['user@example.com', 'create', 'User', '{"id":1}', time])
        subject = Server.new
        subject.store_event(db, {email: 'user@example.com', action: 'create', model: 'User', attrs: {id: 1}}.to_json, time)
      end
    end
  end
end