require 'spec_helper'

class TestClass
  include Blamescope
  
  class << self
    def static_method arg
      'static response'
    end
    blame_method :static_method
  end

  def create arg1, arg2
    'create result'
  end
  blame_method :create

  def update arg1, &block
    return 'block given' if block_given?
  end
  blame_method :update

  def name
    'some name'
  end
end
  
module Blamescope
  describe Trackable do    
    describe "#blame_method" do
      it "should work with multiple arguments" do
        instance = TestClass.new
        expect(Blamescope::Trackable).to receive(:send_event).with(instance, :create)
        expect(instance).to receive(:original_create).with(:arg1,:arg2).and_call_original
        expect(instance.create(:arg1,:arg2)).to eq('create result')
      end

      it "should work with block of code" do
        instance = TestClass.new
        expect(Blamescope::Trackable).to receive(:send_event).with(instance, :update)
        expect(instance).to receive(:original_update).with(:arg1).and_call_original
        expect(instance.update(:arg1){ block }).to eq('block given')
      end
    end

    describe "::blame_method" do
      it "should work with static methods" do
        expect(Blamescope::Trackable).to receive(:send_event).with(TestClass, :static_method)
        expect(TestClass).to receive(:original_static_method).with(:arg1).and_call_original
        expect(TestClass.static_method(:arg1)).to eq('static response')
      end
    end

    describe "#blame_methods" do
      it "should invoke #track_method for each argument" do
        expect(TestClass).to receive(:blame_method).with(:first)
        expect(TestClass).to receive(:blame_method).with(:second)
        TestClass.blame_methods :first, :second
      end
    end
  end
end
