require 'spec_helper'

describe Blamescope do
  describe "#configure" do
    it "should create connection" do
      expect(Blamescope).to receive(:rabbit_connection).with(:config).and_return(exchange: :exchange)
      Blamescope.configure(rabbit: :config)
      expect(Blamescope.exchange).to eq(:exchange)
    end
  end

  describe "#user" do
    it "should get user from warden key" do
      warden = double('warden')
      allow(warden).to receive_message_chain(:user, email: 'user@example.com')
      Thread.current[:env] = {'warden' => warden }
      expect(Blamescope.user).to eq('user@example.com')
    end

    it "should use system user if warden is not available" do
      Thread.current[:env] = nil
      expect(Blamescope.user).to eq("#{ENV['USER']}@#{Socket.gethostname}")
    end
  end

  describe "#emit" do
    it "should publish event" do
      allow(Blamescope).to receive(:user).and_return('user@example.com')
      expect(Blamescope).to receive_message_chain(:exchange, :publish).with({model:'USER', email: 'user@example.com'}.to_json)
      Blamescope.emit(model: 'USER')
    end
  end
end
